'''
Canonical street names!
(To allow name matching between OSM and Portland City DB.)
'''
import re
import string

import pylev

# ratio of edit distance between strings to string length (= max number of edits possible)
def levenshtein_ratio(s1, s2):
    assert len(s1) > 0 and len(s2) > 0
    return pylev.levenshtein(s1, s2) / float(max(len(s1),len(s2)))

# guess at probability names refer to the same street
def match_probability(names1, names2):
    assert len(names1) > 0 and len(names2) > 0
    p = 0

    for name1 in names1:
        can1 = canonical_street_name(name1)
        for name2 in names2:
            can2 = canonical_street_name(name2)
            #print 'DEBUG', name1, name2, can1, can2, pylev.levenshtein(can1,can2)
            #print 'DEBUG ratio:', levenshtein_ratio(can1,can2)
            p = max(p, 1-levenshtein_ratio(can1,can2))

    return p

# rate name match on scale of 0 to 100
def match_score(names1,names2):
    return round(100*match_probability(names1,names2))

# CHARACTER RESTRICTION
# allowed_chars are left unchanged
# ignored_chars are deleted
# characters that are neither in allowed_chars nor in ingored_chars are mapped to space (' ') 
allowed_chars = string.ascii_letters + string.digits + " "  
ignored_chars = "'"

# SINGLE WORD SUBSTITUTIONS 
# applied after words are mapped to all upper case
word_substitutions = {
        # directionals
        'NORTH':'N',
        'SOUTH':'S',
        'EAST':'E',
        'WEST':'W',
        'NORTHEAST':'NE',
        'NORTHWEST':'NW',
        'SOUTHEAST':'SE',
        'SOUTHWEST':'SW',

        # misc
        'JUNIOR':'JR',
        'SAINT':'ST',

        # street types 
        # USPS standard street types -> canonical postal abbreviations (as found int TIGER/Line)
        # Source: http://cpansearch.perl.org/src/SDERLE/Geo-Coder-US-0.21/US/Codes.pm
        'ALLEE':'ALY',
        'ALLEY':'ALY',
        'ALLY':'ALY',
        'ANEX':'ANX',
        'ANNEX':'ANX',
        'ANNX':'ANX',
        'ARCADE':'ARC',
        'AV':'AVE',
        'AVEN':'AVE',
        'AVENU':'AVE',
        'AVENUE':'AVE',
        'AVN':'AVE',
        'AVNUE':'AVE',
        'BAYOO':'BYU',
        'BAYOU':'BYU',
        'BEACH':'BCH',
        'BEND':'BND',
        'BLUF':'BLF',
        'BLUFF':'BLF',
        'BLUFFS':'BLFS',
        'BOT':'BTM',
        'BOTTM':'BTM',
        'BOTTOM':'BTM',
        'BOUL':'BLVD',
        'BOULEVARD':'BLVD',
        'BOULV':'BLVD',
        'BRANCH':'BR',
        'BRDGE':'BRG',
        'BRIDGE':'BRG',
        'BRNCH':'BR',
        'BROOK':'BRK',
        'BROOKS':'BRKS',
        'BURG':'BG',
        'BURGS':'BGS',
        'BYPA':'BYP',
        'BYPAS':'BYP',
        'BYPASS':'BYP',
        'BYPS':'BYP',
        'CAMP':'CP',
        'CANYN':'CYN',
        'CANYON':'CYN',
        'CAPE':'CPE',
        'CAUSEWAY':'CSWY',
        'CAUSWAY':'CSWY',
        'CEN':'CTR',
        'CENT':'CTR',
        'CENTER':'CTR',
        'CENTERS':'CTRS',
        'CENTR':'CTR',
        'CENTRE':'CTR',
        'CIRC':'CIR',
        'CIRCL':'CIR',
        'CIRCLE':'CIR',
        'CIRCLES':'CIRS',
        'CK':'CRK',
        'CLIFF':'CLF',
        'CLIFFS':'CLFS',
        'CLUB':'CLB',
        'CMP':'CP',
        'CNTER':'CTR',
        'CNTR':'CTR',
        'CNYN':'CYN',
        'COMMON':'CMN',
        'CORNER':'COR',
        'CORNERS':'CORS',
        'COURSE':'CRSE',
        'COURT':'CT',
        'COURTS':'CTS',
        'COVE':'CV',
        'COVES':'CVS',
        'CR':'CRK',
        'CRCL':'CIR',
        'CRCLE':'CIR',
        'CRECENT':'CRES',
        'CREEK':'CRK',
        'CRESCENT':'CRES',
        'CRESENT':'CRES',
        'CREST':'CRST',
        'CROSSING':'XING',
        'CROSSROAD':'XRD',
        'CRSCNT':'CRES',
        'CRSENT':'CRES',
        'CRSNT':'CRES',
        'CRSSING':'XING',
        'CRSSNG':'XING',
        'CRT':'CT',
        'CURVE':'CURV',
        'DALE':'DL',
        'DAM':'DM',
        'DIV':'DV',
        'DIVIDE':'DV',
        'DRIV':'DR',
        'DRIVE':'DR',
        'DRIVES':'DRS',
        'DRV':'DR',
        'DVD':'DV',
        'ESTATE':'EST',
        'ESTATES':'ESTS',
        'EXP':'EXPY',
        'EXPR':'EXPY',
        'EXPRESS':'EXPY',
        'EXPRESSWAY':'EXPY',
        'EXPW':'EXPY',
        'EXTENSION':'EXT',
        'EXTENSIONS':'EXTS',
        'EXTN':'EXT',
        'EXTNSN':'EXT',
        'FALLS':'FLS',
        'FERRY':'FRY',
        'FIELD':'FLD',
        'FIELDS':'FLDS',
        'FLAT':'FLT',
        'FLATS':'FLTS',
        'FORD':'FRD',
        'FORDS':'FRDS',
        'FOREST':'FRST',
        'FORESTS':'FRST',
        'FORG':'FRG',
        'FORGE':'FRG',
        'FORGES':'FRGS',
        'FORK':'FRK',
        'FORKS':'FRKS',
        'FORT':'FT',
        'FREEWAY':'FWY',
        'FREEWY':'FWY',
        'FRRY':'FRY',
        'FRT':'FT',
        'FRWAY':'FWY',
        'FRWY':'FWY',
        'GARDEN':'GDN',
        'GARDENS':'GDNS',
        'GARDN':'GDN',
        'GATEWAY':'GTWY',
        'GATEWY':'GTWY',
        'GATWAY':'GTWY',
        'GLEN':'GLN',
        'GLENS':'GLNS',
        'GRDEN':'GDN',
        'GRDN':'GDN',
        'GRDNS':'GDNS',
        'GREEN':'GRN',
        'GREENS':'GRNS',
        'GROV':'GRV',
        'GROVE':'GRV',
        'GROVES':'GRVS',
        'GTWAY':'GTWY',
        'HARB':'HBR',
        'HARBOR':'HBR',
        'HARBORS':'HBRS',
        'HARBR':'HBR',
        'HAVEN':'HVN',
        'HAVN':'HVN',
        'HEIGHT':'HTS',
        'HEIGHTS':'HTS',
        'HGTS':'HTS',
        'HIGHWAY':'HWY',
        'HIGHWY':'HWY',
        'HILL':'HL',
        'HILLS':'HLS',
        'HIWAY':'HWY',
        'HIWY':'HWY',
        'HLLW':'HOLW',
        'HOLLOW':'HOLW',
        'HOLLOWS':'HOLW',
        'HOLWS':'HOLW',
        'HRBOR':'HBR',
        'HT':'HTS',
        'HWAY':'HWY',
        'INLET':'INLT',
        'ISLAND':'IS',
        'ISLANDS':'ISS',
        'ISLES':'ISLE',
        'ISLND':'IS',
        'ISLNDS':'ISS',
        'JCTION':'JCT',
        'JCTN':'JCT',
        'JCTNS':'JCTS',
        'JUNCTION':'JCT',
        'JUNCTIONS':'JCTS',
        'JUNCTN':'JCT',
        'JUNCTON':'JCT',
        'KEY':'KY',
        'KEYS':'KYS',
        'KNOL':'KNL',
        'KNOLL':'KNL',
        'KNOLLS':'KNLS',
        'LA':'LN',
        'LAKE':'LK',
        'LAKES':'LKS',
        'LANDING':'LNDG',
        'LANE':'LN',
        'LANES':'LN',
        'LDGE':'LDG',
        'LIGHT':'LGT',
        'LIGHTS':'LGTS',
        'LNDNG':'LNDG',
        'LOAF':'LF',
        'LOCK':'LCK',
        'LOCKS':'LCKS',
        'LODG':'LDG',
        'LODGE':'LDG',
        'LOOPS':'LOOP',
        'MANOR':'MNR',
        'MANORS':'MNRS',
        'MEADOW':'MDW',
        'MEADOWS':'MDWS',
        'MEDOWS':'MDWS',
        'MILL':'ML',
        'MILLS':'MLS',
        'MISSION':'MSN',
        'MISSN':'MSN',
        'MNT':'MT',
        'MNTAIN':'MTN',
        'MNTN':'MTN',
        'MNTNS':'MTNS',
        'MOTORWAY':'MTWY',
        'MOUNT':'MT',
        'MOUNTAIN':'MTN',
        'MOUNTAINS':'MTNS',
        'MOUNTIN':'MTN',
        'MSSN':'MSN',
        'MTIN':'MTN',
        'NECK':'NCK',
        'ORCHARD':'ORCH',
        'ORCHRD':'ORCH',
        'OVERPASS':'OPAS',
        'OVL':'OVAL',
        'PARKS':'PARK',
        'PARKWAY':'PKWY',
        'PARKWAYS':'PKWY',
        'PARKWY':'PKWY',
        'PASSAGE':'PSGE',
        'PATHS':'PATH',
        'PIKES':'PIKE',
        'PINE':'PNE',
        'PINES':'PNES',
        'PK':'PARK',
        'PKWAY':'PKWY',
        'PKWYS':'PKWY',
        'PKY':'PKWY',
        'PLACE':'PL',
        'PLAIN':'PLN',
        'PLAINES':'PLNS',
        'PLAINS':'PLNS',
        'PLAZA':'PLZ',
        'PLZA':'PLZ',
        'POINT':'PT',
        'POINTS':'PTS',
        'PORT':'PRT',
        'PORTS':'PRTS',
        'PRAIRIE':'PR',
        'PRARIE':'PR',
        'PRK':'PARK',
        'PRR':'PR',
        'RAD':'RADL',
        'RADIAL':'RADL',
        'RADIEL':'RADL',
        'RANCH':'RNCH',
        'RANCHES':'RNCH',
        'RAPID':'RPD',
        'RAPIDS':'RPDS',
        'RDGE':'RDG',
        'REST':'RST',
        'RIDGE':'RDG',
        'RIDGES':'RDGS',
        'RIVER':'RIV',
        'RIVR':'RIV',
        'RNCHS':'RNCH',
        'ROAD':'RD',
        'ROADS':'RDS',
        'ROUTE':'RTE',
        'RVR':'RIV',
        'SHOAL':'SHL',
        'SHOALS':'SHLS',
        'SHOAR':'SHR',
        'SHOARS':'SHRS',
        'SHORE':'SHR',
        'SHORES':'SHRS',
        'SKYWAY':'SKWY',
        'SPNG':'SPG',
        'SPNGS':'SPGS',
        'SPRING':'SPG',
        'SPRINGS':'SPGS',
        'SPRNG':'SPG',
        'SPRNGS':'SPGS',
        'SPURS':'SPUR',
        'SQR':'SQ',
        'SQRE':'SQ',
        'SQRS':'SQS',
        'SQU':'SQ',
        'SQUARE':'SQ',
        'SQUARES':'SQS',
        'STATION':'STA',
        'STATN':'STA',
        'STN':'STA',
        'STR':'ST',
        'STRAV':'STRA',
        'STRAVE':'STRA',
        'STRAVEN':'STRA',
        'STRAVENUE':'STRA',
        'STRAVN':'STRA',
        'STREAM':'STRM',
        'STREET':'ST',
        'STREETS':'STS',
        'STREME':'STRM',
        'STRT':'ST',
        'STRVN':'STRA',
        'STRVNUE':'STRA',
        'SUMIT':'SMT',
        'SUMITT':'SMT',
        'SUMMIT':'SMT',
        'TERR':'TER',
        'TERRACE':'TER',
        'THROUGHWAY':'TRWY',
        'TPK':'TPKE',
        'TR':'TRL',
        'TRACE':'TRCE',
        'TRACES':'TRCE',
        'TRACK':'TRAK',
        'TRACKS':'TRAK',
        'TRAFFICWAY':'TRFY',
        'TRAIL':'TRL',
        'TRAILS':'TRL',
        'TRK':'TRAK',
        'TRKS':'TRAK',
        'TRLS':'TRL',
        'TRNPK':'TPKE',
        'TRPK':'TPKE',
        'TUNEL':'TUNL',
        'TUNLS':'TUNL',
        'TUNNEL':'TUNL',
        'TUNNELS':'TUNL',
        'TUNNL':'TUNL',
        'TURNPIKE':'TPKE',
        'TURNPK':'TPKE',
        'UNDERPASS':'UPAS',
        'UNION':'UN',
        'UNIONS':'UNS',
        'VALLEY':'VLY',
        'VALLEYS':'VLYS',
        'VALLY':'VLY',
        'VDCT':'VIA',
        'VIADCT':'VIA',
        'VIADUCT':'VIA',
        'VIEW':'VW',
        'VIEWS':'VWS',
        'VILL':'VLG',
        'VILLAG':'VLG',
        'VILLAGE':'VLG',
        'VILLAGES':'VLGS',
        'VILLE':'VL',
        'VILLG':'VLG',
        'VILLIAGE':'VLG',
        'VIST':'VIS',
        'VISTA':'VIS',
        'VLLY':'VLY',
        'VST':'VIS',
        'VSTA':'VIS',
        'WALKS':'WALK',
        'WELL':'WL',
        'WELLS':'WLS',
        'WY':'WAY',
                        }

# REGULAR EXPRESSION BASED NAME MAPPINGS
class NameMap(object):
    p = None
    template = None
    def __init__(self,r,template):
        self.p=re.compile(r, re.UNICODE)
        self.template = template
    def apply_map(self,name):
        m = self.p.match(name)
        if m:
            #print 'DEB NameMap.apply name in:',name,'groups:',m.groups()
            name = self.template.format(*m.groups())
            #print 'DEB name out:',name
        return name

mappings = [
        # I84 -> I 84
        NameMap(r"(.*\bI)(\d.*)$",
            "{0} {1}"),

        # I 84 FWY -> I 84
        NameMap(r"(.*\d\s+)(\bFWY\b)(.*)$", 
            "{0}{2}"),
        
        # TRANSIT CTR -> TC
        NameMap(r"(.*)(\bTRANSIT\s+CTR\b)(.*)$",
            "{0}TC{2}"),

        # UNITED STATES -> US
        NameMap(r"(.*)(\bUNITED\s+STATES\b)(.*)$",
            "{0}US{2}"),

        # MC GEE -> MCGEE, LE ROY -> LEROY
        NameMap(r"(.*)\b(MC|LE)\s+(\S+.*)$",
            "{0}{1}{2}"),
        
        # US 30 -> HWY 30,  OR 30 -> HWY 30, etc.
        NameMap(r"(.*)\b(US|OR)\s*(\d.*)$",
            "{0}HWY {2}"),

        ]

def restrict_chars(name):
    global allowed_chars, ingored_chars
    out = []
    for c in name:
        if c in allowed_chars: 
            out.append(c) 
        else:
            if c not in ignored_chars: out.append(' ')
    return ''.join(out)

def _name_to_words(name):
    words=name.strip().split(' ')
    words = [w for w in words if w != '']
    #print "DEBUG words: ", words
    return words

def _words_to_name(words):
    return ' '.join(words).strip()

def _canonical_ws(name):
    words = _name_to_words(name)
    return _words_to_name(words)

def _make_word_substitutions(name):
    global word_substitutions

    words = _name_to_words(name)
    new_words = []
   
    for word in words:
            if word_substitutions.has_key(word):
                    new=word_substitutions[word]
                    if len(new)>0: new_words.append(new)
                    continue
            new_words.append(word)

    return _words_to_name(new_words)

def _apply_mappings(name):
    global mappings
    for nmap in mappings:
        name = nmap.apply_map(name)
    return name

def canonical_street_name(name):
    '''
    Regularizes street name to facilitate comparison.  
    Converts to all caps and applies standard abbreviations.
    '''
    if name is None: return None;

    name = restrict_chars(name)
    name = name.upper()
    name = _make_word_substitutions(name)
    name = _apply_mappings(name)
    name = _canonical_ws(name)

    return name

def test_can(name1,name2=None):
    can1 = canonical_street_name(name1)
    #print "test_can, name1: '%s'  can1: '%s'" % (name1, can1) 
    assert can1 == canonical_street_name(can1)

    if name2:	
	can2 = canonical_street_name(name2)
	#print "test_can, name2: '%s'  can2: '%s'" % (name2, can2) 
	assert can2 == canonical_street_name(can2)
	assert can1 == can2
		
def test():
	test_can("Southeast Sunburst Lane","SE SUNBURST LN")
	test_can("Northeast Saint George Street","NE ST GEORGE ST")
	test_can("NE HALSEY ST FRONTAGE RD","Northeast Halsey Street Frontage Road")
        test_can("I84 FWY","I 84")
        test_can(" 13133","13133")
        test_can("US BANCORP COLUMBIA CENTER TC","UNITED STATES BANCORP COLUMBIA CENTER TRANSIT CENTER")
        test_can("COEUR DALENE DR", "COEUR D'ALENE DR")
        test_can("BURNSIDE BRG", "BURNSIDE BRIDGE")
        test_can("BLAZER TRL","BLAZER TRAIL")
        test_can("HWY 30","US 30")
        test_can("MARTIN LUTHER KING  JUNIOR BLVD","MARTIN LUTHER KING JR BLVD")
        test_can("Mc Gee Street", "McGee Street")
        test_can("Saint Alban's Road", "ST ALBANS RD")
        test_can("Le Roy Avenue", "LeRoy Avenue")

        # 'fuzzy matching' (edit distance)
        assert match_score(['A St'],['B Street']) == 75.0

	print 'cannames PASS'
#doit
if __name__ == "__main__":
    test()



#!/usr/bin/env python

'''
Convert shapefile to geojson
'''

import sys
import itertools
import json 

import geojson
import shapefile #pip pyshp

import geofeatures
import log
import spatialref

def shp2geojson(inFilename,outFileName,clip_rect=None, srs=None):
    # read the shapefile
    reader = shapefile.Reader(inFilename)
    fields = reader.fields[1:]
    fieldNames = [field[0] for field in fields]
    #print 'fieldNames', fieldNames
    features = []
    skipped_no_points = 0
    for (sr, ss) in itertools.izip(reader.iterRecords(), reader.iterShapes()):
        if len(ss.points) == 0: 
            skipped_no_points += 1
            continue  
        atr = dict(zip(fieldNames, sr))
        geom = ss.__geo_interface__
        if not clip_rect or geofeatures.coordinates_intersect_rect_q(geom['coordinates'],clip_rect):
            #print 'DEB geom:', geom
            #features.append(dict(type='Feature', geometry=geom, properties=atr)) 
            features.append(geojson.Feature(geometry=geom, properties=atr)) 

    # log messages
    if skipped_no_points > 0:
        log.warning("Skipped %d shapes in %s because they have no geometry.", skipped_no_points, inFilename)

    # write the geojson file
    geofeatures.write_geojson(features, outFileName, srs=srs)

def test():
    print 'shp2geojson PASS'

if __name__ == '__main__':
    if len(sys.argv) == 3:
        shp2geojson(sys.argv[1],sys.argv[2])
    else:
        print >> sys.stderr, 'Usage: %s foo.shp foo.geojson' % sys.argv[0]



''' log system info before exiting due to uncaught exception '''

import os
import sys

import config
import log

orig_excepthook = None

# log system info before exiting due to uncaught exception.
def crash_info_excepthook(exctype, value, traceback):
    log.critical("""
  --- config.info: 
%s

  --- config.settings:
%s
  """, config.info_str(), config.settings_str())

    orig_excepthook(exctype, value, traceback)

# chain in our excepthook
def init():
    global orig_excepthook
    orig_excepthook = sys.excepthook
    sys.excepthook = crash_info_excepthook




'''
interface to spatialite
'''

import os
import pdb
import tempfile

import pyspatialite
import pyspatialite.dbapi2 

import config
import dbinterface
import log
import spatialref

class Slite(dbinterface.DatabaseInterface):
    def __init__(self, dbName, srid=None, srs=None, verbose=True):
        if not srid:
            srid = config.settings.get('spatialite_srid')
        if not srid and srs:
            srid = srs.spatialite_srid
        if not srid:
            srid = 4326   # longlat, ESPG:4326 

        if not srid: srid = config.settings.get('spatialite_srid')
        super(Slite,self).__init__(dbName, srid=srid, srs=srs, verbose=verbose)        

    # establish database connection 
    def _connect(self):
	con = pyspatialite.dbapi2.connect(self.db_name)
        con.row_factory = dict_factory
	cur = con.cursor()
	return [con, cur]

    def _add_spatial_extension(self):
        if not 'spatial_ref_sys' in self.get_table_names():
            if self.verbose:
                log.info('Initializing spatialite metadata for %s', self.db_name)
            self.exec_sql('select InitSpatialMetadata()')
            self.commit()
        assert 'spatial_ref_sys' in self.get_table_names()

    _interpolation_ref = '?'

    # In general, no guaranteed column order, so return alphabetically sorted
    def get_column_names(self, table):
        rows = self.exec_sql_r("PRAGMA table_info(%s)" % table)
        return sorted([row['name'] for row in rows])

    def get_table_names(self):
        rows = self.exec_sql_r("select name from main.sqlite_master WHERE type='table'")
        return [ row['name'] for row in rows ]

def dict_factory(cursor, row):
    d = {}
    for idx,col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

def testx(fname='out.sqlite'):
    print 'DEB spatialite testx() hack.'
    if os.path.exists(fname): os.remove(fname)

    # srs
    berkeley_url = "http://www.spatialreference.org/ref/epsg/wgs-84-utm-zone-10n/"
    srs=spatialref.SRS(url=berkeley_url)
    print 'srs:', srs

    db = Slite(fname, verbose=False, srs=srs)
    print 'db.srid:', db.srid
    print 'db.srs:', db.srs
    print 'DEB before add'
    db.add_spatial_ref_sys()
    print 'DEB after add'

def test():
    with tempfile.NamedTemporaryFile(suffix='.sqlite', delete=False) as temp:
        fname = temp.name
    try:
        #print 'fname',fname
        db = Slite(fname, verbose=False)
        db.test(verbose=False)
    finally:
        if os.path.exists(fname): os.remove(fname)
    print "spatialite PASS"

if __name__ == "__main__":
    test()
    testx()



#!/usr/bin/env python
'''
Check that spiderosm is installed and functioning properly.
'''

import spiderosm.config
import spiderosm.log
import spiderosm.test

#doit
assert __name__ == "__main__"

#run tests
spiderosm.test.run_tests()

#dump config
spiderosm.log.info('config.info: %s', spiderosm.config.info_str())
spiderosm.log.info('config.settings: %s', spiderosm.config.settings_str())

#we only get here if all the tests pass.
spiderosm.log.info('Congratulations!  SpiderOSM appears to be properly installed and functioning.')





#!/usr/bin/env python
'''
Match Portland Oregon RLIS street centerline to OSM
'''
import os

import spiderosm.centerline
import spiderosm.config
import spiderosm.geo
import spiderosm.log
import spiderosm.match

def _match_city():
    global spiderosm

    # dirs
    project = 'portland'
    gis_data_dir = spiderosm.config.settings.get('gis_data_dir', 'data')
    out_dir = spiderosm.config.settings.get('out_dir', os.path.join('data',project))
    spiderosm.log.info('gis_data_dir=%s out_dir=%s', gis_data_dir, out_dir)
    
    # bounding box
    # if bbox set to None, city data bounding box plus a buffer is used.
    # approximately portland city boundary, with 1 mile buffer.
    portland_bbox = (7604005.67,651319.12,7696978.42,732248.02)
    bbox = spiderosm.geo.buffer_box(portland_bbox,5280) # buffer by 1 mi
    spiderosm.log.info('gis_data_dir=%s out_dir=%s\n  bbox: %s' % (gis_data_dir, out_dir, str(bbox)))
    
    # spatial reference system
    #NAD_1983_HARN_StatePlane_Oregon_North_FIPS_3601_Feet_Intl
    srs = spiderosm.spatialref.SRS(
            url="http://www.spatialreference.org/ref/sr-org/6856/",
            units='feet',
            # proj4text missing at url as of 12/27/2014
            proj4text = '+proj=lcc +lat_1=44.33333333333334 +lat_2=46 +lat_0=43.66666666666666 +lon_0=-120.5 +x_0=2500000 +y_0=0 +datum=NAD83 +units=ft +no_defs',
            spatialite_srid=spiderosm.config.settings.get('spatialite_srid'),
            postgis_srid=spiderosm.config.settings.get('postgis_srid')
            )
    spiderosm.log.info('srs info:\n%s', srs.info())

    # make sure out_dir exists
    if not os.path.exists(out_dir): os.makedirs(out_dir)

    # if a database is enabled write output to it too
    if  spiderosm.config.settings.get('postgis_enabled'):
        import spiderosm.postgis
        db_name = spiderosm.config.settings.get('postgis_dbname') or project
        db = spiderosm.postgis.PGIS(db_name, srs=srs)
        spiderosm.log.info('Results will be written to postgis database %s', db_name)
    elif spiderosm.config.settings.get('spatialite_enabled'):
        import spiderosm.spatialite
        sqlite_fn = os.path.join(out_dir, project + '.sqlite')
        db = spiderosm.spatialite.Slite(sqlite_fn, srs=srs)
        spiderosm.log.info('Results will be written to spatialite database %s', sqlite_fn)
    else:
        db = None

    m = spiderosm.match.Match(
            project=project,
            srs=srs,
            bbox=bbox,
            db=db,
            out_dir=out_dir,

            # CITY DATA (RLIS street data)
            # source: http://rlisdiscovery.oregonmetro.gov/?resourceID=99
            # (No automatic download: go to site manually and download.)
            city_shp= os.path.join(gis_data_dir,'centerline','rlis','streets','streets.shp'),
            city_geojson = os.path.join(out_dir,'streets.geojson'),
            centerline_to_pnwk = spiderosm.centerline.rlis_pnwk
            #UNCOMMENT NEXT LINE TO USE CUSTOMIZED centerline -> pnwk CODE: FURTHER DOWN IN THIS FILE. 
            #centerline_to_pnwk = _city_pnwk,
            #city_network = os.path.join(out_dir,'city'), # .pnwk.geojson

            #OSM DATA (downloaded via overpass API by default)
            #geofabrik extracts updated daily
            #osm_url = 'http://download.geofabrik.de/north-america/us/oregon-latest.osm.pbf',
            #osm = os.path.join(gis_data_dir, 'osm', 'geofabrik.de', 'oregon-latest.osm.pbf'),
            #osm_network = os.path.join(out_dir,'osm'), # .pnwk.geojson

            #OSM BASE (before name fixes)
            #base_url = 'http://download.geofabrik.de/north-america/us/oregon-140901.osm.pbf',
            #base = os.path.join(gis_data_dir,'osm','geofabrik.de','oregon-140901.osm.pbf'),
            #base_network = os.path.join(out_dir,'base') # .pnwk.geojson
            )

    # do the name crosscheck (results written to out_dir)
    m.names_cross_check()
    #m.names_osm_vs_base()

# centerline data -> pnwk
# If you customize this code, be sure to uncomment:
# "m.centerline_to_pnwk = _city_pnwk" in _match_city() above.
# ALSO SEE _city_pnwk() in spiderosm_berkeley.py for an example of FEATURE FILTERING.
def _city_pnwk(features,name=None,props=None,quiet=False,clip_rect=None):
    def rlis_names(feature):
        def _rlis_names0(prefix,street_name,ftype):
            #print 'DEBUG _rlis_names0 prefix,street_name,ftype', prefix,street_name,ftype

            if not street_name or street_name=='UNNAMED': return []
            names = []

            # <prefix> <street_name> <ftype>
            name = ''
            if prefix: name += (prefix + ' ')
            name += street_name
            if ftype: name += (' ' + ftype)
            names.append(name)

            # split out names in RAMPs
            if ftype == 'RAMP':
                for sname in street_name.split('-'):
                    names += _rlis_names0(prefix,sname,'')
                    names += _rlis_names0('',sname,'')  # want 'I205' not 'NE I205'
            
            #print 'DEBUG _rlis_names0 names', names 
            return names
        properties = feature['properties']
        prefix = properties.get('PREFIX')
        street_name = properties.get('STREETNAME')
        ftype = properties.get('FTYPE')
        return _rlis_names0(prefix,street_name,ftype)
    return spiderosm.centerline.make_pnwk(features,
            props=props,
            namesFunc=rlis_names,
            name=name,
            quiet=quiet,
            clip_rect=clip_rect)

#doit
assert __name__ == "__main__"
_match_city()




import json
import os.path
import setuptools


# get package info from SPIDEROSM.json
with open(os.path.join('spiderosm','SPIDEROSM_INFO.json')) as fp:
    spiderosm_info = json.load(fp)

setuptools.setup(name='spiderosm',
      version=spiderosm_info['version'],
      description='GIS conflation tool for matching street networks.',
      long_description='GIS conflation tool: matches segments in one path network (e.g. streets and trails) to corresponding segments in another, based on geography and network connectivity.  Useful, among other things, for combining jurisdictional centerline data with Open Street Maps data.',
      classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: GIS'
      ], 
      keywords='GIS conflation OSM OpenStreetMaps centerline jurisdictional street network matcher',
      url=spiderosm_info['homepage'],
      author=spiderosm_info['author'],
      author_email=spiderosm_info['author_email'],
      license=spiderosm_info['license'],
      packages=['spiderosm'],
      install_requires=[
          'pyproj',
          'shapely', #requires geos library, on Mac: "%brew install geos"  
          'pylev',
          'geojson >= 1.0.9',
          'pyshp' 
          ],
      extras_require={
          'spatialite' : ['pyspatialite >= 3.0.1'],
          'postgis' : 'psycopg2'

          #imposm.parser is now optional:
          #  DOES NOT WORK UNDER WINDOWS!
          #  needs protobuf / protoc, on Mac: "%brew install protobuf --with-python"
          #  needs tokyo-cabinet, on Mac: "%brew install tokyo-cabinet" ?
          },
      include_package_data=True,
      scripts=[
          'spiderosm/bin/spiderosm_test.py',
          'spiderosm/bin/spiderosm_berkeley.py',
          'spiderosm/bin/spiderosm_portland.py'
      ],
      zip_safe=False)
